@@ -29,6 +29,8 @@ except NameError:
 AF_MPLS = 28
 AF_PIPE = 255  # Right now AF_MAX == 40
 DEFAULT_RCVBUF = 16384
+_uuid32 = 0  # (singleton) the last uuid32 value saved to avoid collisions
+_uuid32_lock = threading.Lock()
 
 size_suffixes = {'b': 1,
                  'k': 1024,
@@ -261,16 +263,16 @@ def getbroadcast(addr, mask, family=socket.AF_INET):
     if family == socket.AF_INET:
         i = struct.unpack('>I', i)[0]
         a = 0xffffffff
-        l = 32
+        length = 32
     elif family == socket.AF_INET6:
         i = struct.unpack('>QQ', i)
         i = i[0] << 64 | i[1]
         a = 0xffffffffffffffffffffffffffffffff
-        l = 128
+        length = 128
     else:
         raise NotImplementedError('family not supported')
     # 2. calculate mask
-    m = (a << l - mask) & a
+    m = (a << length - mask) & a
     # 3. calculate default broadcast
     n = (i & m) | a >> mask
     # 4. convert it back to the normal address form
@@ -281,11 +283,16 @@ def getbroadcast(addr, mask, family=socket.AF_INET):
     return socket.inet_ntop(family, n)
 
 
-def dqn2int(mask):
+def dqn2int(mask, family=socket.AF_INET):
     '''
     IPv4 dotted quad notation to int mask conversion
     '''
-    return bin(struct.unpack('>L', socket.inet_aton(mask))[0]).count('1')
+    ret = 0
+    binary = socket.inet_pton(family, mask)
+    for offset in range(len(binary) // 4):
+        ret += bin(struct.unpack('I', binary[offset * 4:
+                                             offset * 4 + 4])[0]).count('1')
+    return ret
 
 
 def hexdump(payload, length=0):
@@ -338,7 +345,7 @@ def load_dump(f, meta=None):
             if a[offset] in (' ', '\t', '\n'):
                 offset += 1
             elif a[offset] == '#':
-                if a[offset:offset+2] == '#!':
+                if a[offset:offset + 2] == '#!':
                     # read and save the code block;
                     # do not parse it here
                     code = ''
@@ -347,11 +354,11 @@ def load_dump(f, meta=None):
                 return data
             elif a[offset] == '\\':
                 # strace hex format
-                data += chr(int(a[offset+2:offset+4], 16))
+                data += chr(int(a[offset + 2:offset + 4], 16))
                 offset += 4
             else:
                 # pyroute2 hex format
-                data += chr(int(a[offset:offset+2], 16))
+                data += chr(int(a[offset:offset + 2], 16))
                 offset += 3
 
     if isinstance(meta, dict):
@@ -530,10 +537,20 @@ def uuid32():
 
     @return: 32bit int uuid
     @rtype: int
+
+    The uuid is guaranteed to be unique within one process.
     '''
-    return fnv1(struct.pack('QQ',
-                            int(time.time() * 1000000),
-                            os.getpid()))
+    global _uuid32
+    global _uuid32_lock
+
+    with _uuid32_lock:
+        candidate = _uuid32
+        while candidate == _uuid32:
+            candidate = fnv1(struct.pack('QQ',
+                                         int(time.time() * 1000000),
+                                         os.getpid()))
+        _uuid32 = candidate
+        return candidate
 
 
 def uifname():
@@ -586,3 +603,15 @@ def metaclass(mc):
                 nvars[k] = v
         return mc(cls.__name__, cls.__bases__, nvars)
     return wrapped
+
+
+def failed_class(message):
+
+    class FailedClass(object):
+
+        def __init__(self, *argv, **kwarg):
+            ret = RuntimeError(message)
+            ret.feature_supported = False
+            raise ret
+
+    return FailedClass
