@@ -13,10 +13,7 @@ from pyroute2.common import dqn2int
 from pyroute2.common import View
 from pyroute2.common import Dotkeys
 from pyroute2.netlink import rtnl
-from pyroute2.netlink import NLM_F_ACK
-from pyroute2.netlink import NLM_F_REQUEST
 from pyroute2.netlink.exceptions import NetlinkError
-from pyroute2.netlink.rtnl import RTM_NEWLINK
 from pyroute2.netlink.rtnl.ifinfmsg import IFF_MASK
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 from pyroute2.ipdb.transactional import Transactional
@@ -39,7 +36,8 @@ supported_kinds = ('bridge',
                    'macvlan',
                    'macvtap',
                    'ipvlan',
-                   'vrf')
+                   'vrf',
+                   'vti')
 
 groups = rtnl.RTMGRP_LINK |\
     rtnl.RTMGRP_NEIGH |\
@@ -52,11 +50,12 @@ def _get_data_fields():
 
     ret = []
     for data in supported_kinds:
-        msg = ifinfmsg.ifinfo.data_map[data]
-        if getattr(msg, 'prefix', None) is not None:
-            ret += [msg.nla2name(i[0]) for i in msg.nla_map]
-        else:
-            ret += [ifinfmsg.nla2name(i[0]) for i in msg.nla_map]
+        msg = ifinfmsg.ifinfo.data_map.get(data)
+        if msg is not None:
+            if getattr(msg, 'prefix', None) is not None:
+                ret += [msg.nla2name(i[0]) for i in msg.nla_map]
+            else:
+                ret += [ifinfmsg.nla2name(i[0]) for i in msg.nla_map]
     return ret
 
 
@@ -665,6 +664,12 @@ class Interface(Transactional):
             for addr in self.ipdb.ipaddr[self['index']]:
                 transaction['ipaddr'].add(addr)
 
+            # Reload the interface data
+            try:
+                self.load_netlink(self.nl.link('get', **request)[0])
+            except Exception:
+                pass
+
         # now we have our index and IP set and all other stuff
         snapshot = self.pick()
 
@@ -751,7 +756,7 @@ class Interface(Transactional):
                         (self.ipdb.interfaces[i]
                          .set_target('master', None)
                          .mirror_target('master', 'link'))
-                        run(nl.link, 'set', index=i, master=0)
+                        run(nl.link, 'update', index=i, master=0)
                     else:
                         transaction.errors.append(KeyError(i))
 
@@ -761,7 +766,7 @@ class Interface(Transactional):
                         (self.ipdb.interfaces[i]
                          .set_target('master', self['index'])
                          .mirror_target('master', 'link'))
-                        run(nl.link, 'set', index=i, master=self['index'])
+                        run(nl.link, 'update', index=i, master=self['index'])
                     else:
                         transaction.errors.append(KeyError(i))
 
@@ -831,8 +836,7 @@ class Interface(Transactional):
                 brequest['kind'] = self['kind']
                 brequest['family'] = AF_BRIDGE
                 wait_all = True
-                run(nl.link, (RTM_NEWLINK, NLM_F_REQUEST | NLM_F_ACK),
-                    **brequest)
+                run(nl.link, 'set', **brequest)
 
             if any([request[item] is not None for item in request]):
                 request['index'] = self['index']
@@ -841,7 +845,7 @@ class Interface(Transactional):
                     request.pop('address')
                     request.pop('broadcast', None)
                 wait_all = True
-                run(nl.link, 'set', **request)
+                run(nl.link, 'update', **request)
                 # Yet another trick: setting ifalias doesn't cause
                 # netlink updates
                 if 'ifalias' in request:
@@ -849,7 +853,7 @@ class Interface(Transactional):
 
             if any([prequest[item] is not None for item in prequest]):
                 prequest['index'] = self['index']
-                run(nl.brport, "set", **prequest)
+                run(nl.brport, 'set', **prequest)
 
             if (wait_all) and (not transaction.partial):
                 transaction.wait_all_targets()
@@ -857,8 +861,7 @@ class Interface(Transactional):
             # 8<---------------------------------------------
             # VLAN flags -- a dirty hack, pls do something with it
             if added.get('vlan_flags') is not None:
-                run(nl.link,
-                    (RTM_NEWLINK, NLM_F_REQUEST | NLM_F_ACK),
+                run(nl.link, 'set',
                     **{'kind': 'vlan',
                        'index': self['index'],
                        'vlan_flags': added['vlan_flags']})
@@ -974,7 +977,7 @@ class Interface(Transactional):
                         request[key] = added[key]
 
                 request['index'] = self['index']
-                run(nl.link, 'set', **request)
+                run(nl.link, 'update', **request)
 
                 countdown = 10
                 while countdown:
@@ -1100,26 +1103,14 @@ class Interface(Transactional):
         '''
         Shortcut: change the interface state to 'up'.
         '''
-        if self['flags'] is None:
-            self['flags'] = 1
-        else:
-            if not self['flags'] & 1:
-                self['flags'] |= 1
-            elif self.current_tx is None:
-                self.begin()
+        self['state'] = 'up'
         return self
 
     def down(self):
         '''
         Shortcut: change the interface state to 'down'.
         '''
-        if self['flags'] is None:
-            self['flags'] = 0
-        else:
-            if self['flags'] & 1:
-                self['flags'] &= ~(self['flags'] & 1)
-            elif self.current_tx is None:
-                self.begin()
+        self['state'] = 'down'
         return self
 
     def remove(self):
