@@ -88,7 +88,7 @@ There are also some useful views, that join `ifinfo` tables with
     (8 rows)
 
 '''
-
+import sys
 import time
 import uuid
 import struct
@@ -110,6 +110,7 @@ from pyroute2.netlink.rtnl.rtmsg import rtmsg
 from pyroute2.netlink.rtnl.rtmsg import nh
 from pyroute2.netlink.rtnl.fibmsg import fibmsg
 from pyroute2.netlink.rtnl.p2pmsg import p2pmsg
+from pyroute2.netlink.rtnl.nsinfmsg import nsinfmsg
 
 # rtnl objects
 from pyroute2.ndb.objects.route import Route
@@ -251,6 +252,7 @@ class DBSchema(object):
                              [(('route_id', ), 'TEXT'),
                               (('nh_id', ), 'INTEGER')])
     spec['rules'] = OrderedDict(fibmsg.sql_schema())
+    spec['netns'] = OrderedDict(nsinfmsg.sql_schema())
     # additional tables
     spec['p2p'] = OrderedDict(p2pmsg.sql_schema())
 
@@ -260,6 +262,7 @@ class DBSchema(object):
                'routes': rtmsg,
                'nh': nh,
                'rules': fibmsg,
+               'netns': nsinfmsg,
                'p2p': p2pmsg}
 
     #
@@ -284,6 +287,7 @@ class DBSchema(object):
                           'RTA_TABLE'),
                'nh': ('route_id',
                       'nh_id'),
+               'netns': ('NSINFO_PATH', ),
                'rules': ('family',
                          'dst_len',
                          'src_len',
@@ -697,20 +701,27 @@ class DBSchema(object):
                 yield row
 
     @publish_exec
-    def export_debug(self, fname='pr2_debug'):
-        if self.rtnl_log:
-            with open(fname, 'w') as f:
-                for table in self.spec.keys():
-                    f.write('\ntable %s\n' % table)
-                    for record in (self
-                                   .execute('SELECT * FROM %s' % table)):
-                        f.write(' '.join([str(x) for x in record]))
-                        f.write('\n')
+    def export(self, fname='stdout'):
+        if fname in ('stdout', 'stderr'):
+            f = getattr(sys, fname)
+        else:
+            f = open(fname, 'w')
+        try:
+            for table in self.spec.keys():
+                f.write('\ntable %s\n' % table)
+                for record in (self
+                               .execute('SELECT * FROM %s' % table)):
+                    f.write(' '.join([str(x) for x in record]))
+                    f.write('\n')
+                if self.rtnl_log:
                     f.write('\ntable %s_log\n' % table)
                     for record in (self
                                    .execute('SELECT * FROM %s_log' % table)):
                         f.write(' '.join([str(x) for x in record]))
                         f.write('\n')
+        finally:
+            if fname not in ('stdout', 'stderr'):
+                f.close()
 
     @publish_exec
     def close(self):
@@ -1016,6 +1027,37 @@ class DBSchema(object):
                           % (self.plch, self.plch, key_query),
                           (gc_mark, target) + route[:-1]))
 
+    def load_nsinfmsg(self, target, event):
+        #
+        # check if there is corresponding source
+        #
+        netns_path = event.get_attr('NSINFO_PATH')
+        if netns_path is None:
+            self.log.debug('ignore %s %s' % (target, event))
+            return
+        if self.ndb._auto_netns:
+            if netns_path.find('/var/run/docker') > -1:
+                source_name = 'docker/%s' % netns_path.split('/')[-1]
+            else:
+                source_name = 'netns/%s' % netns_path.split('/')[-1]
+            if event['header'].get('type', 0) % 2:
+                if source_name in self.ndb.sources.cache:
+                    self.ndb.sources.remove(source_name, code=108, sync=False)
+            elif source_name not in self.ndb.sources.cache:
+                sync_event = None
+                if self.ndb._dbm_autoload and not self.ndb._dbm_ready.is_set():
+                    sync_event = threading.Event()
+                    self.ndb._dbm_autoload.add(sync_event)
+                    self.log.debug('queued event %s' % sync_event)
+                else:
+                    sync_event = None
+                self.log.debug('starting netns source %s' % source_name)
+                self.ndb.sources.async_add(target=source_name,
+                                           netns=netns_path,
+                                           persistent=False,
+                                           event=sync_event)
+        self.load_netlink('netns', target, event)
+
     def load_ndmsg(self, target, event):
         #
         # ignore events with ifindex == 0
@@ -1190,7 +1232,7 @@ class DBSchema(object):
             self.gctime = time.time()
 
             # clean dead snapshots after GC timeout
-            for name, wref in self.snapshots.items():
+            for name, wref in tuple(self.snapshots.items()):
                 if wref() is None:
                     del self.snapshots[name]
                     if name.startswith('ifinfo_'):
@@ -1326,5 +1368,6 @@ def init(ndb, connection, mode, rtnl_log, tid):
                      ifaddrmsg: [partial(ret.load_netlink, 'addresses')],
                      ndmsg: [ret.load_ndmsg],
                      rtmsg: [ret.load_rtmsg],
-                     fibmsg: [partial(ret.load_netlink, 'rules')]}
+                     fibmsg: [partial(ret.load_netlink, 'rules')],
+                     nsinfmsg: [ret.load_nsinfmsg]}
     return ret
