@@ -387,6 +387,7 @@ Please notice, that NLA list *MUST* be mutable.
 '''
 
 import weakref
+import threading
 import traceback
 import logging
 import struct
@@ -565,7 +566,7 @@ NETLINK_TX_RING = 7
 
 NETLINK_LISTEN_ALL_NSID = 8
 
-clean_cbs = {}
+clean_cbs = threading.local()
 
 # Cached results for some struct operations.
 # No cache invalidation required.
@@ -593,11 +594,13 @@ class nlmsg_base(dict):
     nla_map = {}                 # NLA mapping
     sql_constraints = {}
     sql_extra_fields = tuple()
+    sql_extend = tuple()
     nla_flags = 0        # NLA flags
     value_map = {}
     is_nla = False
     prefix = None
     own_parent = False
+    header_type = None
     # caches
     __compiled_nla = False
     __compiled_ft = False
@@ -607,6 +610,7 @@ class nlmsg_base(dict):
     __slots__ = (
         "_buf",
         "data",
+        "chain",
         "offset",
         "length",
         "parent",
@@ -637,6 +641,7 @@ class nlmsg_base(dict):
         self.data = data or bytearray()
         self.offset = offset
         self.length = length or 0
+        self.chain = [self, ]
         if parent is not None:
             # some structures use parents, some not,
             # so don't create cycles without need
@@ -669,8 +674,8 @@ class nlmsg_base(dict):
         ret = []
         for field in cls.fields:
             if field[0][0] != '_':
-                ret.append((field[0],
-                            ' '.join(('INTEGER',
+                ret.append(((field[0], ),
+                            ' '.join(('BIGINT',
                                       cls.sql_constraints.get(field[0], '')))))
         for nla in cls.nla_map:
             if isinstance(nla[0], basestring):
@@ -684,10 +689,17 @@ class nlmsg_base(dict):
             if sql_type:
                 sql_type = ' '.join((sql_type,
                                      cls.sql_constraints.get(nla_name, '')))
-                ret.append((nla_name, sql_type))
+                ret.append(((nla_name, ), sql_type))
+
+        for (fname, ftype) in cls.sql_extra_fields:
+            if isinstance(fname, basestring):
+                fname = (fname, )
+            ret.append((fname, ftype))
+
+        for (dcls, prefix) in cls.sql_extend:
+            for fname, ftype in dcls.sql_schema():
+                ret.append(((prefix, ) + fname, ftype))
 
-        for field in cls.sql_extra_fields:
-            ret.append(field)
         return ret
 
     @property
@@ -722,10 +734,10 @@ class nlmsg_base(dict):
         else:
             # get the msg_seq -- if applicable
             seq = self.get('header', {}).get('sequence_number', None)
-            if seq is not None and seq not in clean_cbs:
-                clean_cbs[seq] = []
+            if seq is not None and seq not in clean_cbs.__dict__:
+                clean_cbs.__dict__[seq] = []
             # attach the callback
-            clean_cbs[seq].append(cb)
+            clean_cbs.__dict__[seq].append(cb)
 
     def unregister_clean_cb(self):
         global clean_cbs
@@ -733,14 +745,14 @@ class nlmsg_base(dict):
         msf = self.get('header', {}).get('flags', 0)
         if (seq is not None) and \
                 (not msf & NLM_F_REQUEST) and \
-                seq in clean_cbs:
-            for cb in clean_cbs[seq]:
+                seq in clean_cbs.__dict__:
+            for cb in clean_cbs.__dict__[seq]:
                 try:
                     cb()
                 except:
                     log.error('Cleanup callback fail: %s' % (cb))
                     log.error(traceback.format_exc())
-            del clean_cbs[seq]
+            del clean_cbs.__dict__[seq]
 
     def _strip_one(self, name):
         for i in tuple(self['attrs']):
@@ -951,7 +963,8 @@ class nlmsg_base(dict):
                     offset += struct.calcsize(fmt)
                 # update length from header
                 # it can not be less than 4
-                self.length = max(self['header']['length'], 4)
+                if 'header' in self:
+                    self.length = max(self['header']['length'], 4)
         # handle the array case
         if self._nla_array:
             self.setvalue([])
@@ -968,7 +981,7 @@ class nlmsg_base(dict):
         else:
             self._ft_decode(self, offset)
 
-        if clean_cbs:
+        if clean_cbs.__dict__:
             self.unregister_clean_cb()
         self.decoded = True
 
@@ -998,12 +1011,16 @@ class nlmsg_base(dict):
 
         # handle the array case
         if self._nla_array:
+            header_type = 1
             for value in self.getvalue():
                 cell = type(self)(data=self.data,
                                   offset=offset,
                                   parent=self)
                 cell._nla_array = False
-                cell['header']['type'] = 1
+                cell['header']['type'] = self.header_type or \
+                    (header_type | self._nla_flags)
+                header_type += 1
+
                 if cell.cell_header is not None:
                     cell.header = cell.cell_header
                 cell.setvalue(value)
@@ -1058,7 +1075,7 @@ class nlmsg_base(dict):
         if self.nla_map:
             offset = self.encode_nlas(offset)
         # calculate the size and write it
-        if self.header is not None:
+        if 'header' in self and self.header is not None:
             self.length = self['header']['length'] = (offset -
                                                       self.offset -
                                                       diff)
@@ -1126,6 +1143,38 @@ class nlmsg_base(dict):
         '''
         return [i[1] for i in self['attrs'] if i[0] == attr]
 
+    def nla(self, attr=None, default=NotInitialized):
+        '''
+        '''
+        if default is NotInitialized:
+            response = nlmsg_base()
+            del response['value']
+            del response['attrs']
+            response.value = None
+        chain = self.get('attrs', [])
+        if attr is not None:
+            chain = [i.nla for i in chain if i.name == attr]
+        else:
+            chain = [i.nla for i in chain]
+        if chain:
+            for link in chain:
+                link.chain = chain
+            response = chain[0]
+        return response
+
+    def __getattribute__(self, key):
+        try:
+            return super(nlmsg_base, self).__getattribute__(key)
+        except AttributeError:
+            if ord(key[0]) < 90:
+                return self.nla(key)
+            raise AttributeError(key)
+
+    def __getitem__(self, key):
+        if isinstance(key, int):
+            return self.chain[key]
+        return dict.__getitem__(self, key)
+
     def __setstate__(self, state):
         return self.load(state)
 
@@ -1453,6 +1502,19 @@ class nla_slot(object):
             return self.cell[1]._nla_flags
         return None
 
+    @property
+    def name(self):
+        return self.cell[0]
+
+    @property
+    def value(self):
+        return self.get_value()
+
+    @property
+    def nla(self):
+        self.try_to_decode()
+        return self.cell[1]
+
     def __getitem__(self, key):
         if key == 1:
             return self.get_value()
