@@ -48,7 +48,7 @@ class IPRuleRequest(IPRequest):
         # now fix the rest
         if 'family' not in self:
             self['family'] = AF_INET
-        if 'priority' not in self:
+        if ('priority' not in self) and ('FRA_PRIORITY' not in self):
             self['priority'] = 32000
         if 'table' in self and 'action' not in self:
             self['action'] = 'to_tbl'
@@ -256,6 +256,9 @@ class IPRouteRequest(IPRequest):
             elif action == 'End.DT6':
                 # Retrieve table
                 table = header['table']
+            elif action == 'End.DT4':
+                # Retrieve table
+                table = header['table']
             elif action == 'End.B6':
                 # Parse segs
                 segs = header['srh']['segs']
@@ -364,6 +367,9 @@ class IPRouteRequest(IPRequest):
             elif isinstance(value, int):
                 dict.__setitem__(self, key, {'label': value,
                                              'bos': 1})
+            elif value == '':
+                # ignore empty values for src/dst
+                return
             elif value != 'default':
                 value = value.split('/')
                 mask = None
