@@ -5,5 +5,9 @@ parent = TC_H_ROOT
 
 
 class options(nla):
-        fields = (('bands', 'i'),
-                  ('priomap', '16B'))
+    fields = (('bands', 'i'),
+              ('priomap', '16B'))
+
+
+def get_parameters(kwarg):
+    return kwarg
