@@ -1,4 +1,5 @@
 import os
+import errno
 import atexit
 import pickle
 import select
@@ -11,6 +12,7 @@ from io import BytesIO
 from socket import SOL_SOCKET
 from socket import SO_RCVBUF
 from pyroute2 import config
+from pyroute2 import netns as netnsmod
 from pyroute2.netlink.nlsocket import NetlinkMixin
 if config.uname[0][-3:] == 'BSD':
     from pyroute2.iproute.bsd import IPRoute
@@ -55,31 +57,44 @@ class Transport(object):
         ret = pickle.load(dump)
         return ret
 
-    def recv(self):
+    def _m_recv(self, own_queue, other_queue, check):
         while self.run:
-            with self.lock:
-                if not self.brd_queue.empty():
-                    return self.brd_queue.get()
+            if self.lock.acquire(False):
                 try:
-                    ret = self.__recv()
-                except struct.error:
                     try:
-                        return self.brd_queue.get(timeout=5)
+                        ret = own_queue.get(False)
+                        if ret is None:
+                            continue
+                        else:
+                            return ret
                     except queue.Empty:
-                        raise Exception('I/O error')
-                if ret['stage'] == 'broadcast':
+                        pass
+                    ret = self.__recv()
+                    if not check(ret['stage']):
+                        other_queue.put(ret)
+                    else:
+                        other_queue.put(None)
+                        return ret
+                finally:
+                    self.lock.release()
+            else:
+                ret = None
+                try:
+                    ret = own_queue.get(timeout=1)
+                except queue.Empty:
+                    pass
+                if ret is not None:
                     return ret
-                self.cmd_queue.put(ret)
+
+    def recv(self):
+        return self._m_recv(self.brd_queue,
+                            self.cmd_queue,
+                            lambda x: x == 'broadcast')
 
     def recv_cmd(self):
-        while self.run:
-            with self.lock:
-                if not self.cmd_queue.empty():
-                    return self.cmd_queue.get()
-                ret = self.__recv()
-                if ret['stage'] != 'broadcast':
-                    return ret
-                self.brd_queue.put(ret)
+        return self._m_recv(self.cmd_queue,
+                            self.brd_queue,
+                            lambda x: x != 'broadcast')
 
     def close(self):
         self.run = False
@@ -97,7 +112,7 @@ class ProxyChannel(object):
                                  'error': None})
 
 
-def Server(trnsp_in, trnsp_out):
+def Server(trnsp_in, trnsp_out, netns=None):
 
     def stop_server(signum, frame):
         Server.run = False
@@ -106,6 +121,8 @@ def Server(trnsp_in, trnsp_out):
     signal.signal(signal.SIGTERM, stop_server)
 
     try:
+        if netns is not None:
+            netnsmod.setns(netns)
         ipr = IPRoute()
         lock = ipr._sproxy.lock
         ipr._s_channel = ProxyChannel(trnsp_out, 'broadcast')
@@ -229,6 +246,8 @@ class RemoteSocket(NetlinkMixin):
         msg = None
         while True:
             msg = self.trnsp_in.recv()
+            if msg is None:
+                raise EOFError()
             if msg['stage'] == 'signal':
                 os.kill(os.getpid(), msg['data'])
             else:
@@ -246,7 +265,7 @@ class RemoteSocket(NetlinkMixin):
             except ValueError:
                 pass
 
-    def close(self):
+    def close(self, code=errno.ECONNRESET):
         with self.shutdown_lock:
             if not self.closed:
                 super(RemoteSocket, self).close()
@@ -254,8 +273,8 @@ class RemoteSocket(NetlinkMixin):
                 self._cleanup_atexit()
                 self.trnsp_out.send({'stage': 'shutdown'})
                 # send loopback nlmsg to terminate possible .get()
-                if self.remote_trnsp_out is not None:
-                    data = struct.pack('IHHQIQQ', 28, 2, 0, 0, 104, 0, 0)
+                if code > 0 and self.remote_trnsp_out is not None:
+                    data = struct.pack('IHHQIQQ', 28, 2, 0, 0, code, 0, 0)
                     self.remote_trnsp_out.send({'stage': 'broadcast',
                                                 'data': data,
                                                 'error': None})
